﻿using UnityEngine;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace core
{
    public class Game_Object : MonoBehaviour
    {
        private const string EntireGamePrefab_c = "Prefabs/EntireGame";
        private GameObject entireGame;

        [SerializeField]
        private UnityEngine.UI.Text scoreText;
        private int score = 0;
        public int Score
        {
            get { return this.score; }
            private set
            {
                this.score = value;
                this.scoreText.text = "Score - " + this.score.ToString("000");
            }
        }
        public void AddToScore(int amount)
        {
            this.Score = Math.Max(0, this.score + amount);
        }

        [SerializeField]
        private UnityEngine.UI.Text timerText;
        [SerializeField]
        private CameraAnimator mainCamera;
        [SerializeField]
        private MainMenuController mainMenu;
        [SerializeField]
        private AudioSource musTheme;
        private bool hasStarted = false;
        public bool IsGameOver() { return !this.hasStarted; }
        [SerializeField]
        private float timerMax = 120.0f;
        private float timeRemaining;

        public void Update()
        {
            if (!this.hasStarted)
                return;

            this.TickTimer(Time.deltaTime);
        }

        private void TickTimer(float t)
        {
            this.timeRemaining -= t;
            if (this.timeRemaining < 0.0f) {
                this.EndToMainMenu();
            }

            this.timerText.text = "Time - " + Mathf.CeilToInt(this.timeRemaining).ToString("000");
        }

        public void PrepareFromMainMenu()
        {
            if (this.entireGame != null) {
                MonoBehaviour.Destroy(this.entireGame);
                MonoBehaviour.Destroy(this.scoreText.gameObject);
                MonoBehaviour.Destroy(this.timerText.gameObject);
                this.scoreText = null;
                this.timerText = null;
                var sushis = GameObject.FindObjectsOfType<sushi.SushiController>();
                foreach (var sushi in sushis) {
                    MonoBehaviour.Destroy(sushi.gameObject);
                }
                this.entireGame = null;
            }
            this.entireGame = MonoBehaviour.Instantiate(Resources.Load(EntireGamePrefab_c)) as GameObject;
            // Hard-coding with paths; since the previous text objects aren't actually destroyed until the end of the frame, they can still be retrieved with GameObject.Find.
            this.scoreText = this.entireGame.transform.Find("SignScore/Cube/Canvas/ScoreText").GetComponent<UnityEngine.UI.Text>();
            this.timerText = this.entireGame.transform.Find("SignTimer/Cube/Canvas/TimerText").GetComponent<UnityEngine.UI.Text>();
        }
        public void StartFromMainMenu()
        {
            this.hasStarted = true;
            this.score = 0;
            this.timeRemaining = timerMax;
            this.mainCamera.Animate(false);
            this.timeRemaining = this.timerMax;
            this.musTheme.Play();
        }
        public void EndToMainMenu()
        {
            this.hasStarted = false;
            this.mainMenu.UpdateForGameEnd(this.score);
            this.mainCamera.Animate(true);
            this.musTheme.Stop();
        }
    }

    public static class Game
    {
        public static Game_Object game;

        static Game() {
            game = GameObject.FindObjectOfType<Game_Object>();
        }
    }
}
