﻿using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;

namespace core
{
    /// <summary>
    /// Controls the main menu.
    /// </summary>
    public class MainMenuController : MonoBehaviour
    {
        // Defined in a few places
        private delegate void Callback_VoidVoid();

        [SerializeField]
        private List<ControlIconController> controlIcons;
        private int controlIconsWaitingFor;
        [SerializeField]
        private UnityEngine.UI.Text instructionsText;
        [SerializeField]
        private UnityEngine.UI.Text gameEndText;

        public void Start() {
            this.Initialize();
        }

        public void Initialize()
        {
            this.controlIconsWaitingFor = this.controlIcons.Count;
            for (int i = 0; i < this.controlIcons.Count; i++) {
                this.controlIcons[i].PrepareToReady(this.OnControlIconReady);
            }
        }

        private void OnControlIconReady()
        {
            this.controlIconsWaitingFor--;
            if (this.controlIconsWaitingFor <= 0) {
                core.Game.game.PrepareFromMainMenu(); // If this is done as part of DelayThenDo, causes a crash. I don't know why.
                this.StartCoroutine(this.DelayThenDo(core.Game.game.StartFromMainMenu));
            }
        }

        private IEnumerator DelayThenDo(Callback_VoidVoid thingToDo)
        {
            yield return new WaitForSeconds(0.5f);
            thingToDo();
        }

        public void UpdateForGameEnd(int score)
        {
            this.instructionsText.gameObject.SetActive(false);
            this.gameEndText.gameObject.SetActive(true);
            this.gameEndText.text = string.Format(
@"Well done! Your score was {0}.

Play again?", score.ToString("000"));
            this.StartCoroutine(this.DelayThenDo(this.Initialize)); // Delay in case the player is still giving input
        }
    }
}
