﻿using UnityEngine;
using System.Collections;

namespace player
{
    /// <summary>
    /// Corresponding to one joystick, moves the hitbox for the tentacle.
    /// </summary>
    public class TentacleController : MonoBehaviour
    {
        [SerializeField]
        private Vector3 initialLocalPosition;
        [SerializeField]
        private Vector3 swingLocalPosition;
        [SerializeField]
        private float swingTime;

        [SerializeField]
        private string inputAxis;
        [SerializeField]
        private Animator animator;
        [SerializeField]
        private AudioSource sfxTentacle;

        // For use in starting the game from the main menu. Ensures that all tentacle input is prepared by the player.
        public bool HasMoved { get; private set; } // Default false

        private bool isSwinging = false;
        /// <summary>
        /// Not accurately named, but if a sushi collides with this collider, it should check this to see if it gets hit up (true) or down.
        /// </summary>
        public bool IsSwinging
        {
            get { return this.isSwinging; }
            private set
            {
                this.isSwinging = value;
                if (this.isSwinging) {
                    this.StartCoroutine(this.Swing());
                }
            }
        }

        public void Update()
        {
            if (core.Game.game.IsGameOver())
                return; // Not disabling because order of events is spawn -> start game, so would disable just after spawning.

            if (this.IsSwinging)
                return;

            this.IsSwinging = UnityEngine.Input.GetAxis(this.inputAxis) != 0.0f;
        }

        private IEnumerator Swing()
        {
            this.HasMoved = true;
            this.animator.SetBool("isSwinging", true);
            this.animator.GetComponent<TentacleAnimator>().OnIdleStateReached = this.OnSwingEnd;
            this.sfxTentacle.Play();

            this.transform.localPosition = this.swingLocalPosition;
            yield return new WaitForSeconds(0.2f); // Time for which the hit box is out.
            this.transform.localPosition = this.initialLocalPosition;
        }

        private void OnSwingEnd()
        {
            this.IsSwinging = false;
            this.animator.SetBool("isSwinging", false);
        }
       
    }

}