﻿using UnityEngine;
using System;
using System.Collections;
using System.Linq;
using System.Text;

namespace sushi
{
    /// <summary>
    /// Controls a single sushi object.
    /// </summary>
    [RequireComponent(typeof(Rigidbody))]
    public class SushiController : MonoBehaviour
    {
        private SushiSpawner spawner;
        private float speedY;
        private Rigidbody myRigidbody;
        private GameObject currentTarget = null;

        [SerializeField]
        private AudioSource sfxSushi;

        public void Initialize(SushiSpawner spawner, float speedY) {
            this.spawner = spawner;
            this.speedY = speedY;
            this.transform.position = spawner.transform.position;
            this.myRigidbody = this.GetComponent<Rigidbody>();
            this.SetNewTrajectory();
        }

        private void SetNewTrajectory()
        {
            this.currentTarget = this.spawner.GetNewTarget(this.currentTarget); // NOTE: Ignores whichever tentacle actually hit this sushi. Not really a problem.
            // Quadratic equation!
            float a = 0.5f * Physics.gravity.y;
            float b = this.speedY;
            float c = this.transform.position.y - this.currentTarget.transform.position.y;
            float x = (-b - Mathf.Sqrt(Mathf.Pow(b, 2) - 4 * a * c)) / (2 * a);
            myRigidbody.velocity = new Vector3((currentTarget.transform.position.x - this.transform.position.x) / x, this.speedY, 0.0f);
            myRigidbody.AddTorque(0.0f, 0.0f, UnityEngine.Random.Range(-2f, 2f), ForceMode.Impulse);
        }

        public void OnTriggerEnter(Collider other) // Other is a collider of some sort hitting this, which is a trigger, so OnTriggerEnter...?
        {
            player.TentacleController tentacle = other.gameObject.GetComponent<player.TentacleController>();
            if (tentacle != null) {
                if (tentacle.IsSwinging) {
                    this.SetNewTrajectory();
                    this.StartCoroutine(this.DisableCollisionBriefly());
                } else {
                    this.myRigidbody.velocity = new Vector3(0.0f, Physics.gravity.y, 5.0f); // No longer possible
                }
                this.sfxSushi.Play();
            }
            else if (other.gameObject.GetComponent<TableController>() != null) {
                if (!other.gameObject.GetComponent<TableController>().TryGivingSushi(this)) {
                    this.myRigidbody.velocity = new Vector3(0.0f, 0.0f, -3.0f);
                }
            }
        }

        public IEnumerator DisableCollisionBriefly()
        {
            Collider collider = this.GetComponent<Collider>();
            collider.enabled = false;
            yield return new WaitForSeconds(1.0f);
            collider.enabled = true;
        }

    }
}
