﻿using UnityEngine;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace sushi
{
    /// <summary>
    /// Spawns and manages sushi.
    /// </summary>
    public class SushiSpawner : MonoBehaviour
    {
        private const string sushiPrefabPath_c = "Prefabs/sushi";

        [SerializeField]
        private List<GameObject> targetObjects;
        [SerializeField]
        private KeyCode inputButton;
        [SerializeField]
        private KeyCode inputButtonAlt;
        [SerializeField]
        private AudioSource sfxLeftCtrl;
        // For serialization
        public Vector2 minAndMaxSushiSpeeds;

        /// <summary>
        /// Gets a new target for a sushi.
        /// </summary>
        /// <param name="previousTarget">Previous target or null</param>
        public GameObject GetNewTarget(GameObject previousTarget)
        {
            int index = this.targetObjects.FindIndex((GameObject go) => { return go == previousTarget; }); // Returns -1 if not found (so if previousTarget is null)
            int newIndex = UnityEngine.Random.Range(index + 1, Math.Min(this.targetObjects.Count, index + 6)); // Exception if at the end of the list already, which I'll allow, since that case should be handled elsewhere.
            return this.targetObjects[newIndex];
        }

        public void Update()
        {
            if (core.Game.game.IsGameOver())
                return;

            if (Input.GetKeyDown(this.inputButton) || Input.GetKeyDown(this.inputButtonAlt)) {
                this.SpawnSushi();
            }
        }

        private GameObject SpawnSushi()
        {
            GameObject newSushiObject = MonoBehaviour.Instantiate(Resources.Load(sushiPrefabPath_c)) as GameObject;
            float sushiSpeedY = UnityEngine.Random.Range(minAndMaxSushiSpeeds.x, minAndMaxSushiSpeeds.y);
            newSushiObject.GetComponent<SushiController>().Initialize(this, sushiSpeedY);
            this.sfxLeftCtrl.Play();
            return newSushiObject;
        }
    }
}
